# Generated by Django 2.2.15 on 2020-08-18 20:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('canvas', '0002_auto_20200806_1632'),
    ]

    operations = [
        migrations.DeleteModel(
            name='CanvasGlobalConfiguration',
        ),
    ]
